--===========
--Exercise 1.	List work items at the project level
--===========
SELECT *
FROM ip.structure
WHERE structure_name = '$Plan'
	AND depth = (
		SELECT option_value
		FROM ip.global_options
		WHERE option_id = '1'
		)

--===========
--Exercise 2.	List resource items at the resource level
--===========
SELECT *
FROM ip.structure
WHERE structure_name = '$Res'
	AND detailed = 'N'

--===========
--Exercise 3.	List descriptions of work at the project level
--===========
SELECT stree.*
	,s1.description
FROM ip.structure_tree stree
JOIN ip.structure s1 ON s1.structure_code = anc_code
WHERE stree.dsc_code = '1209'
ORDER BY stree.anc_depth

--===========
--Exercise 4.	List descriptions of work at a level below project
--===========
SELECT ptree.*
	,s1.description
	,s1.depth
	,s1.place
FROM ip.project_tree ptree
JOIN ip.structure s1 ON s1.structure_code = ptree.dsc_code
WHERE ptree.anc_code = '1209'
ORDER BY s1.depth
	,s1.place

--List projects at a specific level
SELECT ptree.*
	,s1.description
FROM ip.project_tree ptree
JOIN ip.structure s1 ON s1.structure_code = ptree.dsc_code
WHERE ptree.anc_depth = '7'

--List all activities in a project using the PROJECT_TREE table
SELECT s1.description, s1.depth, s1.place
FROM ip.project_tree ptree
JOIN ip.structure s1 on ptree.dsc_code = s1.structure_code
WHERE ptree.anc_code = '1489'

--Order activities using the STRUCTURE_MAP table
SELECT s1.description, s1.depth, s1.place, smap.map_code
FROM ip.project_tree ptree
JOIN ip.structure s1 on ptree.dsc_code = s1.structure_code
JOIN ip.structure_map smap on ptree.dsc_code = smap.structure_code
WHERE ptree.anc_code = '1489'
ORDER BY smap.map_code

--===========
--Exercise 5.	List resources in the order displayed in Planview Enterprise
--===========
SELECT *
FROM ip.structure s1
JOIN ip.structure_map smap ON s1.structure_code = smap.structure_code
WHERE s1.structure_name = '$Res'
ORDER BY smap.map_code

--"Climb-up" or "Climb-down" a structure using the STRUCTURE_MAP 
SELECT *
FROM structure s1
JOIN structure_map pr ON s1.structure_code = pr.structure_code
--join structure_map ch on ch.map_code like pr.map_code + '%' --climb up
--join structure_map ch on pr.map_code like ch.map_code + '%' --climb down
WHERE ch.structure_code = '<structure_code>'
ORDER BY pr.map_code 

--List the number of levels within a structure
SELECT *
FROM structure_level
WHERE structure_name = '$Plan'
ORDER BY level_num

--===========
--Exercise 6.	List all values in the resource structure with level descriptions
--===========
SELECT slvl.NAME
	,s1.description
	,s1.depth
	,s1.place
FROM structure s1
JOIN structure_level slvl ON s1.structure_name = slvl.structure_name
	AND s1.depth = slvl.level_num
JOIN structure_map smap ON s1.structure_code = smap.structure_code
WHERE s1.structure_name = '$Res'
ORDER BY smap.map_code

--===========
--Exercise 7.	Use the PLANNING_ENTITY table to list all activities of a project
--===========
SELECT s2.description 'ProjectName'
	, s1.description 'ActivityName'
FROM planning_entity pe
JOIN structure_map smap ON pe.planning_code = smap.structure_code
JOIN structure s1 ON pe.planning_code = s1.structure_code
JOIN structure s2 ON pe.ppl_code = s2.structure_code
WHERE pe.ppl_code = '1489'
ORDER BY smap.map_code 

--===========
--Exercise 8.	Identify the project-level information from an activity
--===========
SELECT s1.description 'ActivityName'
	,s2.description 'ProjectName'
FROM planning_entity pe
JOIN structure s1 ON pe.planning_code = s1.structure_code
JOIN structure s2 ON pe.ppl_code = s2.structure_code
WHERE pe.planning_code = '1018'

--===========
--Exercise 9.	List projects and activities based on the project start date
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ActivityName'
FROM planning_entity pe
JOIN structure_map smap ON pe.planning_code = smap.structure_code
WHERE pe.ppl_code IN (
		SELECT ppl_code
		FROM planning_entity
		WHERE actual_start >= '06/01/2016'
		)
ORDER BY smap.map_code

--===========
--Exercise 10.	List projects and milestones based on the project actual or schedule start dates
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'MilestoneName'
	,isnull(pe.actual_start, pe.schedule_start) 'MilestoneDueDate'
FROM planning_entity pe
JOIN structure_map smap ON pe.planning_code = smap.structure_code
WHERE pe.milestone = 'Y'
ORDER BY smap.map_code 

--===========
--Exercise 11.	List projects and activities with work statuses
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ActivityName'
	,(SELECT description
		FROM structure
		WHERE structure_code = pe.code20
		) 'WorkStatus'
FROM planning_entity pe
JOIN structure_map smap ON pe.planning_code = smap.structure_code
WHERE pe.ppl_code IN (
		SELECT ppl_code
		FROM planning_entity
		WHERE actual_start >= '06/01/2016'
		)
ORDER BY smap.map_code

--===========
--Exercise 12.	List alternate structures that are stored in the PLANNING_ENTITY table
--===========
SELECT s1.description
	,sinfo.map_table
	,sinfo.map_column
	,sinfo.map_key
	,sinfo.structure_type
	,sinfo.enabled
	,sinfo.keep_history
	,sinfo.ppl_only_ind
FROM structure_info sinfo
JOIN structure s1 ON sinfo.alt_structure = s1.structure_code
WHERE sinfo.map_table = 'planning_entity'

--===========
--Exercise 13.	List projects and activities with project name, activity name, actual dates, 
				--schedule dates, work status, work type, and internal priority
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ActivityName'
	,pe.actual_start 'ActualStartDate'
	,pe.actual_finish 'ActualFinishDate'
	,pe.schedule_start 'ScheduleStartDate'
	,pe.schedule_finish 'ScheduleFinishDate'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code20
		) 'WorkStatus'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code22
		) 'WorkType'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code26
		) 'InternalPriority'
FROM planning_entity pe
JOIN structure_map smap ON pe.planning_code = smap.structure_code
WHERE pe.ppl_code = '1489'
ORDER BY smap.map_code

--Which tables store information for work fields?
SELECT  map_table, count(*)
FROM field_info
WHERE prim_structure = '$Plan'
GROUP BY map_table

--===========
--Exercise 14.	Add investment risk and priority score to query exercise 12
--===========
--List the field_ids, where, and how the fields are stored
SELECT description
	,map_table
	,map_column
	,map_key
	,field_type
	,field_format
	,prim_structure
	,ppl_only_ind
FROM field_info
WHERE description IN (
		'Investment Risk Score'
		,'Investment Priority Score'
		)

--Exercise 14.	Solution
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ActivityName'
	,pe.actual_start 'ActualStartDate'
	,pe.actual_finish 'ActualFinishDate'
	,pe.schedule_start 'ScheduleStartDate'
	,pe.schedule_finish 'ScheduleFinishDate'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code20
		) 'WorkStatus'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code22
		) 'WorkType'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code26
		) 'InternalPriority'
	,cdata.score1 'Investment Priority Score'
	,cdata.score2 'Investment Risk Score'
FROM planning_entity pe
JOIN structure_map smap ON pe.planning_code = smap.structure_code
JOIN custom_data cdata on pe.planning_code = cdata.planning_code
WHERE pe.ppl_code = '1489'
ORDER BY smap.map_code

--===========
--Exercise 15.	List incomplete projects with work status, start date, and work description
--===========
--Identify structure_code for work statuses
SELECT *
FROM structure
WHERE structure_name = 'Wbs20'

--Exercise 15: Solution
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code20
		) 'WorkStatus'
	,ISNULL(actual_start, schedule_start) 'ProjectStartDate'
	,ltxt.line_text 'WorkDetailDescription'
	,ltxt.line_number
FROM planning_entity pe
JOIN long_text ltxt ON pe.planning_code = ltxt.key2
WHERE pe.code20 <> 'WBS20$CLOS'
	AND pe.planning_code = pe.ppl_code
	AND ltxt.key1 = 'PE01'

--===========
--Exercise 16.	List projects with entry date, requested dates, schedule dates, actual dates, 
				--and variances between the requested and actual start dates
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ProjectName'
	,pinfo.entry_date 'EntryDate'
	,pinfo.requested_start 'RequestedStartDate'
	,pinfo.requested_finish 'RequestedFinishDate'
	,pe.schedule_start 'ScheduleStartDate'
	,pe.schedule_finish 'ScheduleFinishDate'
	,pe.actual_start 'ActualStartDate'
	,pe.actual_finish 'ActualFinishDate'
	,ISNULL(datediff(day, requested_start, actual_start), 0) 'DateDiffDays'
	,ISNULL(ip.pfn_cal_duration(pe.calendar, requested_start, actual_start), 0) / (
		SELECT average_day
		FROM ip.time_unit
		WHERE calendar = pe.calendar
		) 'Pfn_calDiffDays'
FROM planning_entity pe
JOIN project_info pinfo ON pe.planning_code = pinfo.planning_code
WHERE pe.planning_code = pe.ppl_code

--List where and how multiple-select work alternate structure assignments are stored
SELECT *
FROM structure_info
WHERE structure_type = 'M'
	AND prim_structure = '$Plan'

--===========
--Exercise 17.	List projects with business capabilities
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = mattrib.attribute_code
		) 'BusinessCapabilities'
	,mattrib.percentage_share
	,mattrib.numeric_value
	,mattrib.text_value
	,mattrib.primary_value_ind
	,mattrib.place
FROM planning_entity pe
JOIN multi_attribute mattrib ON pe.planning_code = mattrib.prime_structure_code
WHERE mattrib.alt_structure_name = '$Cap'

--===========
--Exercise 18.	List resources with resource name, resource calendar, and employment dates
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = resource_code
		) 'ResourceName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = calendar
		) 'ResourceCalendar'
	,start_date 'EmploymentStart'
	,term_date 'EmploymentEnd'
FROM resources

--List where and how resource alternate structures are stored
SELECT alt_structure
	,(
		SELECT description
		FROM structure
		WHERE structure_code = alt_structure
		) 'AltStructureDescription'
	,map_table
	,map_column
	,map_key
	,structure_type
FROM structure_info
WHERE prim_structure = '$Res'

--List where and how resource fields are stored
SELECT field_id
	,description 'FieldDescription'
	,map_table
	,map_column
	,map_key
	,field_type
FROM field_info
WHERE prim_structure = '$Res'

--===========
--Exercise 19.	List resources with resource department, location, and resource type
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = stree.anc_code
		) 'ResourceName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = res.resource_code
		) 'ResourceName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = cri1.father_code
		) 'Location'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = cri2.father_code
		) 'ResourceType'
FROM resources res
LEFT OUTER JOIN criteria cri1 ON res.resource_code = cri1.resource_code
	AND cri1.structure_name = 'Loc'
LEFT OUTER JOIN criteria cri2 ON res.resource_code = cri2.resource_code
	AND cri2.structure_name = 'ResType'
JOIN STRUCTURE_TREE stree ON res.resource_code = stree.dsc_code
	AND stree.anc_depth = '4'
JOIN STRUCTURE_MAP smap ON res.resource_code = smap.structure_code
ORDER BY smap.map_code

--===========
--Exercise 21.	List projects in the Requested status with project entry date, name, 
				--and user role of the user whom had created the project
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = res.resource_code
		) 'ResourceName'
	,iusr.full_name 'AssociatedUserName'
FROM resources res
LEFT OUTER JOIN ip_user iusr ON res.logon_id = iusr.user_name


--Exercise 21.	Solution
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code20
		) 'WorkStatus'
	,pinfo.entry_date 'EntryDate'
	,iusr.full_name 'ProjectCreatedBy'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = iusr.role_code
		) 'ProjectCreatorUserRole'
FROM planning_entity pe
JOIN project_info pinfo ON pe.planning_code = pinfo.planning_code
JOIN ip_user iusr ON pinfo.created_by = iusr.user_name
WHERE pe.code20 = 'WBS20$OPEN'

--===========
--Exercise 22.	List projects in the Open/Approved status with project entry date, name, 
				--and user role of the user who created the project
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.planning_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = pe.code20
		) 'WorkStatus'
	,pinfo.entry_date 'EntryDate'
	,iusr.full_name 'ProjectCreatedBy'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = iusr.role_code
		) 'ProjectCreatorUserRole'
	,(
		SELECT full_name
		FROM ip_user
		WHERE user_name = wft.user_name
		) 'LifecycleProjectManager'
FROM planning_entity pe
JOIN project_info pinfo ON pe.planning_code = pinfo.planning_code
JOIN ip_user iusr ON pinfo.created_by = iusr.user_name
LEFT OUTER JOIN workflow wf ON pe.planning_code = wf.planning_code
LEFT OUTER JOIN wf_team wft ON wf.workflow_id = wft.workflow_id
	AND wft.lifecycle_role_code = (
		SELECT structure_code
		FROM structure
		WHERE structure_name = 'LfCycRole'
			AND description = 'Project Manager'
		)
WHERE pe.code20 = 'WBS20$OPEN'

--===========
--Exercise 23.	List time entry information with resource name, project name, activity name 
				--or standard activity name, period start, timesheet status, reported hours, 
				--and the name of the user who submitted, approved, or disapproved the timesheet
--===========
SELECT sres.description 'ResourceName'
	,sproj.description 'ProjectName'
	,sact.description 'ActivityName'
	,prd.period_start 'PeriodStart'
	,tr.integrate_status 'TimesheetStatus'
	,tr.reported 'ReportedHrs'
	,tr.signed_by 'SubmittedBy'
	,tr.approved_by 'Approved/DisapprovedBy'
	,tr.type
FROM time_reported tr
JOIN user_period prd ON tr.period_number = prd.period_number
JOIN structure sres ON tr.resource_code = sres.structure_code
JOIN structure sact ON tr.activity_code = sact.structure_code
CROSS APPLY (
	SELECT pe.ppl_code
	FROM planning_entity pe
	WHERE pe.planning_code = tr.activity_code
		AND sact.structure_name = '$Plan'
	
	UNION ALL
	
	SELECT s.structure_code 'ppl_code'
	FROM structure s
	WHERE sact.structure_name = 'StdAct'
		AND s.structure_code = sact.structure_name
	) proj
JOIN structure sproj ON proj.ppl_code = sproj.structure_code

--===========
--Exercise 24.	List actual effort hours by resources, activity name, and project name or standard activity name
--===========
SELECT isnull((
			SELECT description
			FROM structure
			WHERE structure_code = pe.ppl_code
			), 'Standard Activity') 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = tr.activity_code
		) 'ActivityName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = tr.resource_code
		) 'ResourceName'
	,sum(daily_effort) / 60000.0 'ActualEffortHrs'
FROM daily_tr tr
JOIN planning_entity pe ON tr.activity_code = pe.planning_code
WHERE tr.integrate_status = 'I'
GROUP BY tr.activity_code
	,tr.resource_code
	,pe.ppl_code

--===========
--Exercise 25.	List remaining effort hours by assignment type, resources, activity name, and project name
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = sch.activity_code
		) 'ActivityName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = sch.resource_code
		) 'ResourceName'
	,sum(sch.daily_effort) / 60000.0 'AllocationRemainHrs'
	,sum(sch.daily_effort_rsv) / 60000.0 'ReserveRemainHrs'
	,sum(sch.daily_effort_req) / 60000.0 'RequirementRemainHrs'
	,sum(sch.daily_effort + sch.daily_effort_rsv + sch.daily_effort_req) / 60000.0 'TotalRemainHrs'
FROM daily_sch sch
JOIN planning_entity pe ON sch.activity_code = pe.planning_code
GROUP BY sch.activity_code
	,sch.resource_code
	,pe.ppl_code

--Use DateDiff, DateName, Min and Max functions on daily tables
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = sch.activity_code
		) 'ActivityName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = sch.resource_code
		) 'ResourceName'
	,0.0 'ActualEffortHrs'
	,sum(sch.daily_effort) / 60000.0 'AllocationRemainHrs'
	,sum(sch.daily_effort_rsv) / 60000.0 'ReserveRemainHrs'
	,sum(sch.daily_effort_req) / 60000.0 'RequirementRemainHrs'
	,datediff(month, 0, sch.day_date) 'MonthIndex'
	,datename(month, min(sch.day_date)) 'Month'
	,datename(year, min(sch.day_date)) 'Year'
	,min(sch.day_date) 'RangeStart'
	,max(sch.day_date) 'RangeEnd'
FROM daily_sch sch
JOIN planning_entity pe ON sch.activity_code = pe.planning_code
GROUP BY sch.activity_code
	,sch.resource_code
	,pe.ppl_code
	,datediff(month, 0, sch.day_date)

--Use Union All to combine both actual and scheduled effort from daily tables
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = sch.activity_code
		) 'ActivityName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = sch.resource_code
		) 'ResourceName'
	,0.0 'ActualEffortHrs'
	,sum(sch.daily_effort) / 60000.0 'AllocationRemainHrs'
	,sum(sch.daily_effort_rsv) / 60000.0 'ReserveRemainHrs'
	,sum(sch.daily_effort_req) / 60000.0 'RequirementRemainHrs'
	,datediff(month, 0, sch.day_date) 'MonthIndex'
	,datename(month, min(sch.day_date)) 'Month'
	,datename(year, min(sch.day_date)) 'Year'
	,min(sch.day_date) 'RangeStart'
	,max(sch.day_date) 'RangeEnd'
FROM daily_sch sch
JOIN planning_entity pe ON sch.activity_code = pe.planning_code
GROUP BY sch.activity_code
	,sch.resource_code
	,pe.ppl_code
	,datediff(month, 0, sch.day_date)
UNION ALL
SELECT isnull((
			SELECT description
			FROM structure
			WHERE structure_code = pe.ppl_code
			), 'Standard Activity') 'ProjectName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = tr.activity_code
		) 'ActivityName'
	,(
		SELECT description
		FROM structure
		WHERE structure_code = tr.resource_code
		) 'ResourceName'
	,sum(daily_effort) / 60000.0 'ActualEffortHrs'
	,0.0 'AllocationRemainHrs'
	,0.0 'ReserveRemainHrs'
	,0.0 'RequirementRemainHrs'
	,datediff(month, 0, tr.day_date) 'MonthIndex'
	,datename(month, min(tr.day_date)) 'Month'
	,datename(year, min(tr.day_date)) 'Year'
	,min(tr.day_date) 'RangeStart'
	,max(tr.day_date) 'RangeEnd'
FROM daily_tr tr
JOIN planning_entity pe ON tr.activity_code = pe.planning_code
WHERE tr.integrate_status = 'I'
GROUP BY tr.activity_code
	,tr.resource_code
	,pe.ppl_code
	,datediff(month, 0, tr.day_date)

--===========
--Exercise 26.	List projects in a work portfolio with work status, project start 
				--and finish dates, total actual effort hours, and total remaining effort hours
--===========
SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,0.0 'ActualEffortHrs'
	,sum(sch.daily_effort + sch.daily_effort_rsv + sch.daily_effort_req) / 60000.0 'RemainingEffortHrs'
FROM daily_sch sch
JOIN planning_entity pe ON sch.activity_code = pe.planning_code
WHERE pe.ppl_code IN (
		SELECT structure_code
		FROM portal_filter
		WHERE filter_code = '7209'
		)
GROUP BY pe.ppl_code

UNION ALL

SELECT (
		SELECT description
		FROM structure
		WHERE structure_code = pe.ppl_code
		) 'ProjectName'
	,sum(daily_effort) / 60000.0 'ActualEffortHrs'
	,0.0 'RemainingEffortHrs'
FROM daily_tr tr
JOIN planning_entity pe ON tr.activity_code = pe.planning_code
WHERE tr.integrate_status = 'I'
	AND pe.ppl_code IN (
		SELECT structure_code
		FROM portal_filter
		WHERE filter_code = '7209'
		)
GROUP BY pe.ppl_code

--Use @StructureCode to develop datasets for reporting
FROM daily_tr tr
JOIN planning_entity pe ON tr.activity_code = pe.planning_code
WHERE tr.integrate_status = 'I'
	AND pe.ppl_code IN (
		SELECT structure_code
		FROM portal_filter
		WHERE filter_code = @StructureCode
		)
GROUP BY pe.ppl_code





